==> patch-4.1.7-ONex-OMS-pro-source-PHP5.3.tar.gz: ShopexErp/app/siso/lib/receipt/sales/sold.php <==
<?php
class siso_receipt_sales_sold extends siso_receipt_sales_abstract implements siso_receipt_sales_interface{

    /**
     *
     * 根据参数组织订单销售的销售单数据
     * @param array $params
     */
    public function get_sales_data($params){
        $order_original_data = array();
        $sales_data = array();
        $delivery_id = $params['delivery_id'];

        $deliveryObj = app::get('ome')->model('delivery');
        $orderIds = $deliveryObj->getOrderIdsByDeliveryIds(array($delivery_id));
        $db = kernel::database();
        $ome_original_dataLib = kernel::single('ome_sales_original_data');
        $ome_sales_dataLib = kernel::single('ome_sales_data');
        foreach ($orderIds as $key => $orderId){
            //判断订单是否已经生成过销售单
            $sales_detail = $db->selectrow("SELECT sale_id FROM sdb_ome_sales WHERE order_id=".$orderId);
            if ($sales_detail) return false;
            $order_original_data = $ome_original_dataLib->init($orderId);
            if($order_original_data){
                $sales_data[$orderId] = $ome_sales_dataLib->generate($order_original_data,$delivery_id);
                if(!$sales_data[$orderId]){
                    return false;
                }
            }else{
                return false;
            }
            unset($order_original_data);
        }

        //平摊预估物流运费，主要处理订单合并发货以及多包裹单的运费问题
        $ome_sales_logistics_feeLib = kernel::single('ome_sales_logistics_fee');
        $ome_sales_logistics_feeLib->calculate($orderIds,$sales_data);

        $this->format_sales_items($sales_data,$delivery_id);
        //重新转换结构将PKG的转换成平铺结构

        $this->tran_sales_items($sales_data);

        return $sales_data;

    }

    /**
    * 余单撤销数据组织
    *
    */
    private function format_sales_items(&$sales_data,$delivery_id){

        $orderObj = app::get('ome')->model('orders');
        $delivery_items_detailObj = app::get('ome')->model('delivery_items_detail');

        foreach ($sales_data as $order_id=>$sales){
            $order_items    = $detail_items = $temp_arr = array();
            $orders = $orderObj->dump($order_id,'process_status');
            if (in_array($orders['process_status'],array('remain_cancel'))){

                $sql    = "SELECT a.item_id, a.obj_id, a.nums, a.sendnum, a.item_type, b.quantity
                            FROM sdb_ome_order_items AS a LEFT JOIN sdb_ome_order_objects AS b ON a.obj_id=b.obj_id
                            WHERE a.order_id='".$order_id."'";
                $temp_arr   = kernel::database()->select($sql);
                foreach ($temp_arr as $key => $val){
                    $item_id    = $val['item_id'];

                    $val['dly_nums']        = intval($val['sendnum'] / $val['nums'] * $val['quantity']);//已发货数量
                    $order_items[$item_id]  = $val;
                }


                $temp_arr   = $delivery_items_detailObj->getList('item_detail_id, order_item_id, order_obj_id, number', array('order_id'=>$order_id));
                foreach ($temp_arr as $key => $val)
                {
                    $item_detail_id     = $val['item_detail_id'];
                    $order_item_id      = $val['order_item_id'];

                    $val['dly_nums']    = intval($order_items[$order_item_id]['dly_nums']);//总发货数量

                    $detail_items[$item_detail_id]  = $val;
                }
                foreach ($sales['sales_items'] as $key => &$item){
                    $item_detail_id     = $item['item_detail_id'];
                    $nums               = intval($item['nums']);//购买数量
                    $send_num           = $detail_items[$item_detail_id]['dly_nums'];//发货数量

                    $item['pmt_price']      = round(($item['pmt_price'] / $nums * $send_num), 2);
                    $item['sale_price']     = round(($item['sale_price'] / $nums * $send_num), 2);
                    $item['sales_amount']   = round(($item['sales_amount'] / $nums * $send_num), 2);
                    $item['nums']           = $send_num;
                }
            }
        }
    }

    /**
     *
     * 针对类型是PKG的做转换结构
     */

    public function tran_sales_items(&$sales_data){
        $salesBasicMaterialObj    = app::get('material')->model('sales_basic_material');
        $orderItemObj             = app::get('ome')->model('order_items');
        $orderObjects             = app::get('ome')->model('order_objects');
        foreach ($sales_data as $order_id=>$sales){
            $tmp_sale_items = array();
            foreach ($sales['sales_items'] as $key => $val){
                if($val['obj_type']=='pkg')
                {


                    $getItem    = $orderObjects->dump(array('order_id'=>$order_id, 'bn'=>$val['bn'], 'obj_type'=>'pkg'), 'obj_id, goods_id, obj_type');

                    $order_items    = $orderItemObj->getList('item_id, product_id, bn, name, price, pmt_price, sale_price, amount, nums', array('obj_id'=>$getItem['obj_id']));
                    $items_count    =  count($order_items);
                    $delivery_items = $this->get_delivery_detail($order_id);

                    $apportion_flag = false;
                    if($val['apportion_pmt'])
                    {
                        #关联基础物料贡献占比
                        $salesBasicMList = array();
                        $tempData        = $salesBasicMaterialObj->getList('bm_id, rate', array('sm_id'=>$getItem['goods_id']));
                        foreach ($tempData as $sKey => $sVal)
                        {
                            $salesBasicMList[$sVal['bm_id']]    = $sVal['rate'];
                        }
                        #平摊优惠
                        $bm_rate    =   $apportion_pmt  = $pmt_price    =   $item_id=  0;

                        $apportion_flag = true;
                    }
                    foreach($order_items as $item){

                        if ( $apportion_flag ){
                            $item_i++;
                            if($item_i == $items_count)
                            {
                                $item['apportion_pmt']    = $val['apportion_pmt'] - $apportion_pmt;
                            }
                            else
                            {
                                $bm_rate    = $salesBasicMList[$item['product_id']] / 100;
                                $item['apportion_pmt']    = round(($val['apportion_pmt'] * $bm_rate), 2);
                                $apportion_pmt  += $item['apportion_pmt'];
                            }
                        }else{
                            $item['apportion_pmt']  =   0;
                        }
                        $item['sales_amount']   =   $item['amount'] - $item['apportion_pmt'];
                        $item['obj_type']       =   $val['obj_type'];
                        $item['orginal_price']  =   $item['price'];
                        $item['sales_material_bn'] =   $val['sales_material_bn'];
                        $item['item_detail_id'] =   $delivery_items[$getItem['obj_id']][$item['item_id']]['item_detail_id'];

                        $key_i = $item['item_id'];
                        unset($item['item_id']);
                        $tmp_sale_items[$key_i] = $item;
                    }

                    unset($sales_data[$order_id]['sales_items'][$key]);

                }
            }

            $sales_data[$order_id]['sales_items'] = array_merge($sales_data[$order_id]['sales_items'],$tmp_sale_items);

        }

    }

/**
 * @ 根据ID获取delivery_items_detail LIST
 * @DateTime  2018-03-02T13:51:41+0800
 * @param    $order_id
 * @return    array
 */
    public function get_delivery_detail($order_id)
    {
        static $delivery_items = array();

        if ($delivery_items[$order_id]) return $delivery_items[$order_id];

        $delivery_items_detailObj = app::get('ome')->model('delivery_items_detail');
        $item_list   = $delivery_items_detailObj->getList('order_id,item_detail_id, order_item_id, order_obj_id, number', array('order_id'=>$order_id));
        foreach($item_list as $item){
            $delivery_items[$item['order_id']][$item['order_obj_id']][$item['order_item_id']]   =   $item;
        }

        return $delivery_items[$order_id];
    }
}

==> patch-4.1.8-ONex-OMS-pro-source-PHP5.6.tar.gz: ShopexErp/app/siso/lib/receipt/sales/sold.php <==
<?php
class siso_receipt_sales_sold extends siso_receipt_sales_abstract implements siso_receipt_sales_interface{

    /**
     *
     * 根据参数组织订单销售的销售单数据
     * @param array $params
     */
    public function get_sales_data($params){
        $order_original_data = array();
        $sales_data = array();
        $delivery_id = $params['delivery_id'];

        $deliveryObj = app::get('ome')->model('delivery');
        $orderIds = $deliveryObj->getOrderIdsByDeliveryIds(array($delivery_id));
        $db = kernel::database();
        $ome_original_dataLib = kernel::single('ome_sales_original_data');
        $ome_sales_dataLib = kernel::single('ome_sales_data');
        foreach ($orderIds as $key => $orderId){
            //判断订单是否已经生成过销售单
            $sales_detail = $db->selectrow("SELECT sale_id FROM sdb_ome_sales WHERE order_id=".$orderId);
            if ($sales_detail) return false;
            $order_original_data = $ome_original_dataLib->init($orderId);
            if($order_original_data){
                $sales_data[$orderId] = $ome_sales_dataLib->generate($order_original_data,$delivery_id);
                if(!$sales_data[$orderId]){
                    return false;
                }
            }else{
                return false;
            }
            unset($order_original_data);
        }

        //平摊预估物流运费，主要处理订单合并发货以及多包裹单的运费问题
        $ome_sales_logistics_feeLib = kernel::single('ome_sales_logistics_fee');
        $ome_sales_logistics_feeLib->calculate($orderIds,$sales_data);

        $this->format_sales_items($sales_data,$delivery_id);
        //重新转换结构将PKG的转换成平铺结构

        $this->tran_sales_items($sales_data);

        return $sales_data;

    }

    /**
    * 余单撤销数据组织
    *
    */
    private function format_sales_items(&$sales_data,$delivery_id){

        $orderObj = app::get('ome')->model('orders');
        $delivery_items_detailObj = app::get('ome')->model('delivery_items_detail');

        foreach ($sales_data as $order_id=>$sales){
            $order_items    = $detail_items = $temp_arr = array();
            $orders = $orderObj->dump($order_id,'process_status');
            if (in_array($orders['process_status'],array('remain_cancel'))){

                $sql    = "SELECT a.item_id, a.obj_id, a.nums, a.sendnum, a.item_type, b.quantity
                            FROM sdb_ome_order_items AS a LEFT JOIN sdb_ome_order_objects AS b ON a.obj_id=b.obj_id
                            WHERE a.order_id='".$order_id."'";
                $temp_arr   = kernel::database()->select($sql);
                foreach ($temp_arr as $key => $val){
                    $item_id    = $val['item_id'];

                    $val['dly_nums']        = intval($val['sendnum'] / $val['nums'] * $val['quantity']);//已发货数量
                    $order_items[$item_id]  = $val;
                }


                $temp_arr   = $delivery_items_detailObj->getList('item_detail_id, order_item_id, order_obj_id, number', array('order_id'=>$order_id));
                foreach ($temp_arr as $key => $val)
                {
                    $item_detail_id     = $val['item_detail_id'];
                    $order_item_id      = $val['order_item_id'];

                    $val['dly_nums']    = intval($order_items[$order_item_id]['dly_nums']);//总发货数量

                    $detail_items[$item_detail_id]  = $val;
                }
                foreach ($sales['sales_items'] as $key => &$item){
                    $item_detail_id     = $item['item_detail_id'];
                    $nums               = intval($item['nums']);//购买数量
                    $send_num           = $detail_items[$item_detail_id]['dly_nums'];//发货数量

                    $item['pmt_price']      = round(($item['pmt_price'] / $nums * $send_num), 2);
                    $item['sale_price']     = round(($item['sale_price'] / $nums * $send_num), 2);
                    $item['sales_amount']   = round(($item['sales_amount'] / $nums * $send_num), 2);
                    $item['nums']           = $send_num;
                }
            }
        }
    }

    /**
     *
     * 针对类型是PKG和LKB的做转换结构
     */

    public function tran_sales_items(&$sales_data){
        $salesBasicMaterialObj    = app::get('material')->model('sales_basic_material');
        $orderItemObj             = app::get('ome')->model('order_items');
        $orderObjects             = app::get('ome')->model('order_objects');
        foreach ($sales_data as $order_id=>$sales){
            $tmp_sale_items = array();
            foreach ($sales['sales_items'] as $key => $val){
                if($val['obj_type']=='pkg')
                {


                    $getItem    = $orderObjects->dump(array('order_id'=>$order_id, 'bn'=>$val['bn'], 'obj_type'=>'pkg'), 'obj_id, goods_id, obj_type');

                    $order_items    = $orderItemObj->getList('item_id, product_id, bn, name, price, pmt_price, sale_price, amount, nums', array('obj_id'=>$getItem['obj_id']));
                    $items_count    =  count($order_items);
                    $delivery_items = $this->get_delivery_detail($order_id);

                    $apportion_flag = false;
                    if($val['apportion_pmt'])
                    {
                        #关联基础物料贡献占比
                        $salesBasicMList = array();
                        $tempData        = $salesBasicMaterialObj->getList('bm_id, rate', array('sm_id'=>$getItem['goods_id']));
                        foreach ($tempData as $sKey => $sVal)
                        {
                            $salesBasicMList[$sVal['bm_id']]    = $sVal['rate'];
                        }
                        #平摊优惠
                        $bm_rate    =   $apportion_pmt  = $pmt_price    =   $item_id=  0;

                        $apportion_flag = true;
                    }
                    foreach($order_items as $item){

                        if ( $apportion_flag ){
                            $item_i++;
                            if($item_i == $items_count)
                            {
                                $item['apportion_pmt']    = $val['apportion_pmt'] - $apportion_pmt;
                            }
                            else
                            {
                                $bm_rate    = $salesBasicMList[$item['product_id']] / 100;
                                $item['apportion_pmt']    = round(($val['apportion_pmt'] * $bm_rate), 2);
                                $apportion_pmt  += $item['apportion_pmt'];
                            }
                        }else{
                            $item['apportion_pmt']  =   0;
                        }
                        $item['sales_amount']   =   $item['amount'] - $item['apportion_pmt'];
                        $item['obj_type']       =   $val['obj_type'];
                        $item['orginal_price']  =   $item['price'];
                        $item['sales_material_bn'] =   $val['bn'];
                        $item['item_detail_id'] =   $delivery_items[$getItem['obj_id']][$item['item_id']]['item_detail_id'];

                        $key_i = $item['item_id'];
                        unset($item['item_id']);
                        $tmp_sale_items[$key_i] = $item;
                    }

                    unset($sales_data[$order_id]['sales_items'][$key]);

                }elseif($val['obj_type'] == 'lkb'){
                    $getItem = $orderObjects->dump(array('order_id'=>$order_id, 'bn'=>$val['bn'], 'obj_type'=>'lkb'), 'obj_id, goods_id, obj_type');
                    $order_items = $orderItemObj->getList('item_id, product_id, bn, name, price, pmt_price, sale_price, amount, nums', array('obj_id'=>$getItem['obj_id']));
                    $delivery_items = $this->get_delivery_detail($order_id);
                    foreach($order_items as $item){
                        $item['apportion_pmt'] = 0;
                        $item['sales_amount'] = $item['amount'] - $item['apportion_pmt'];
                        $item['obj_type'] = $val['obj_type'];
                        $item['orginal_price'] = $item['price'];
                        $item['sales_material_bn'] = $val['bn'];
                        $item['item_detail_id'] = $delivery_items[$getItem['obj_id']][$item['item_id']]['item_detail_id'];
                        $key_i = $item['item_id'];
                        unset($item['item_id']);
                        $tmp_sale_items[$key_i] = $item;
                    }
                    unset($sales_data[$order_id]['sales_items'][$key]);
                }
            }

            $sales_data[$order_id]['sales_items'] = array_merge($sales_data[$order_id]['sales_items'],$tmp_sale_items);

        }

    }

/**
 * @ 根据ID获取delivery_items_detail LIST
 * @DateTime  2018-03-02T13:51:41+0800
 * @param    $order_id
 * @return    array
 */
    public function get_delivery_detail($order_id)
    {
        static $delivery_items = array();

        if ($delivery_items[$order_id]) return $delivery_items[$order_id];

        $delivery_items_detailObj = app::get('ome')->model('delivery_items_detail');
        $item_list   = $delivery_items_detailObj->getList('order_id,item_detail_id, order_item_id, order_obj_id, number', array('order_id'=>$order_id));
        foreach($item_list as $item){
            $delivery_items[$item['order_id']][$item['order_obj_id']][$item['order_item_id']]   =   $item;
        }

        return $delivery_items[$order_id];
    }
}
